Module: AmritkScalarTest::Internal::Type::Union Private
- Includes:
- Converter, Util::SorbetRuntimeSupport
- Defined in:
- lib/amritk-scalar-test/internal/type/union.rb,
sig/amritk-scalar-test/internal/type/union.rbs
This module is part of a private API. You should avoid using this module if possible, as it may be removed or be changed in the future.
Class Method Summary collapse
- .derefed_variants ⇒ ::Array[[Symbol?, top, ::Hash[Symbol, top]]] private
- .variants ⇒ ::Array[top] private
Instance Method Summary collapse
- #==(other) ⇒ Boolean
- #===(other) ⇒ Boolean
-
#coerce(value, state:) ⇒ Object
private
Tries to efficiently coerce the given value to one of the known variants.
- #dump(value, state:) ⇒ Object private
- #hash ⇒ Integer
- #inspect(depth: 0) ⇒ String private
- #to_sorbet_type ⇒ Object private
-
#variants ⇒ Array<Object>
private
All of the specified variants for this union.
Methods included from Util::SorbetRuntimeSupport
#const_missing, #define_sorbet_constant!, #sorbet_constant_defined?, to_sorbet_type
Methods included from Converter
coerce, dump, inspect, meta_info, new_coerce_state, type_info
Class Method Details
.derefed_variants ⇒ ::Array[[Symbol?, top, ::Hash[Symbol, top]]]
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
10 |
# File 'sig/amritk-scalar-test/internal/type/union.rbs', line 10
def self.derefed_variants: -> ::Array[[Symbol?, top, ::Hash[Symbol, top]]]
|
.variants ⇒ ::Array[top]
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
12 |
# File 'sig/amritk-scalar-test/internal/type/union.rbs', line 12
def self.variants: -> ::Array[top]
|
Instance Method Details
#==(other) ⇒ Boolean
110 111 112 |
# File 'lib/amritk-scalar-test/internal/type/union.rb', line 110 def ==(other) AmritkScalarTest::Internal::Type::Union === other && other.derefed_variants == derefed_variants end |
#===(other) ⇒ Boolean
99 100 101 102 103 |
# File 'lib/amritk-scalar-test/internal/type/union.rb', line 99 def ===(other) known_variants.any? do |_, variant_fn| variant_fn.call === other end end |
#coerce(value, state:) ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Tries to efficiently coerce the given value to one of the known variants.
If the value cannot match any of the known variants, the coercion is considered non-viable and returns the original value.
141 142 143 144 145 146 147 148 149 150 151 152 153 154 155 156 157 158 159 160 161 162 163 164 165 166 167 168 169 170 171 172 173 174 175 176 177 178 |
# File 'lib/amritk-scalar-test/internal/type/union.rb', line 141 def coerce(value, state:) if (target = resolve_variant(value)) return AmritkScalarTest::Internal::Type::Converter.coerce(target, value, state: state) end strictness = state.fetch(:strictness) exactness = state.fetch(:exactness) alternatives = [] known_variants.each do |_, variant_fn| target = variant_fn.call exact = state[:exactness] = {yes: 0, no: 0, maybe: 0} state[:branched] += 1 coerced = AmritkScalarTest::Internal::Type::Converter.coerce(target, value, state: state) yes, no, maybe = exact.values if (no + maybe).zero? || (!strictness && yes.positive?) exact.each { exactness[_1] += _2 } state[:exactness] = exactness return coerced elsif maybe.positive? alternatives << [[-yes, -maybe, no], exact, coerced] end end case alternatives.sort_by!(&:first) in [] exactness[:no] += 1 state[:error] = ArgumentError.new("no matching variant for #{value.inspect}") value in [[_, exact, coerced], *] exact.each { exactness[_1] += _2 } coerced end .tap { state[:exactness] = exactness } ensure state[:strictness] = strictness end |
#dump(value, state:) ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
189 190 191 192 193 194 195 196 197 198 199 200 |
# File 'lib/amritk-scalar-test/internal/type/union.rb', line 189 def dump(value, state:) if (target = resolve_variant(value)) return AmritkScalarTest::Internal::Type::Converter.dump(target, value, state: state) end known_variants.each do target = _2.call return AmritkScalarTest::Internal::Type::Converter.dump(target, value, state: state) if target === value end super end |
#hash ⇒ Integer
117 |
# File 'lib/amritk-scalar-test/internal/type/union.rb', line 117 def hash = variants.hash |
#inspect(depth: 0) ⇒ String
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
225 226 227 228 229 230 231 232 233 234 |
# File 'lib/amritk-scalar-test/internal/type/union.rb', line 225 def inspect(depth: 0) if depth.positive? return is_a?(Module) ? super() : self.class.name end members = variants.map { AmritkScalarTest::Internal::Type::Converter.inspect(_1, depth: depth.succ) } prefix = is_a?(Module) ? name : self.class.name "#{prefix}[#{members.join(' | ')}]" end |
#to_sorbet_type ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
205 206 207 208 209 210 211 212 213 214 215 |
# File 'lib/amritk-scalar-test/internal/type/union.rb', line 205 def to_sorbet_type types = variants.map { AmritkScalarTest::Internal::Util::SorbetRuntimeSupport.to_sorbet_type(_1) }.uniq case types in [] T.noreturn in [type] type else T.any(*types) end end |
#variants ⇒ Array<Object>
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
All of the specified variants for this union.
29 |
# File 'lib/amritk-scalar-test/internal/type/union.rb', line 29 def variants = derefed_variants.map { _2 } |